#!/usr/bin/env python
# coding: utf-8

# In[ ]:


from ..process import *
from . import branch as _branch

def ls(): return lines('git','remote')

def exists(name): return name in ls()

def url(name, *args, **kwargs):
    if not exists(name):
        if len(args) == 1:
            default = args[0]
        elif 'default' in kwargs:
            default = kwargs.pop('default')
        else:
            raise ValueError(f"Remote {name} doesn't exist")

        return default
    
    return line('git','remote','get-url',name)
_url = url

def init(name, url, branch=None, remote_branch=None, fetch=True, checkout=True, push=True):
    if exists(name):
        existing_url = _url(name)
        if existing_url != url:
            run('git','remote','set-url',name,url)
    else:
        run('git','remote','add',name,url)
    
    if fetch:
        run('git','fetch',name)
    
    if branch:
        remote_branch = remote_branch or 'master'
        upstream = f'{name}/{remote_branch}'
        try:
            run('git','branch','-u',upstream,branch)
        except CalledProcessError as e:
            if push:
                print(f'Failed to track upstream branch {upstream}; attempting to push {branch}:{remote_branch}')
                run('git','push',name,f'{branch}:{remote_branch}')
                run('git','branch','-u',upstream,branch)
            else:
                raise e
        if checkout:
            run('git','checkout',branch)

def push(name=None, local=None, remote=None):
    refspec = None
    if local:
        if remote:
            refspec = f'{local}:{remote}'
        else:
            refspec = f'{local}:{local}'
    
    run('git','push',name,refspec)




try:
    from git import Repo, Git, InvalidGitRepositoryError
    from functools import partial
    import os
    def make_repo(*args, exist_ok=False, search_parent_directories=True, gitignore=None, msg=None, **kwargs):
        load_kwargs = kwargs.copy()
        load_kwargs['search_parent_directories'] = search_parent_directories
        if exist_ok:
            try:
                return Repo(*args, **load_kwargs)
            except InvalidGitRepositoryError:
                repo = Repo.init(*args, **kwargs)
                if gitignore:
                    if isinstance(gitignore, str): gitignore = [ gitignore ]
                    gitignore_path = os.path.join(repo.working_dir, '.gitignore')
                    with open(gitignore_path, 'w') as f:
                        for line in gitignore:
                            f.write(line + '\n')
                    repo.git.add('.gitignore')
                    msg = msg or 'initial commit'
                    repo.git.commit('-m',msg)
                return repo
        else:
            return Repo(*args, **load_kwargs)
except ImportError:
    pass


from . import branch, clone, diff, head, remote, submodule
from .head import fmt, sha
from .remote import push
from ..process import run


from ._checkout import Checkout
checkout = Checkout()


def merge(ref, msg=None, ff=None):
    cmd = ['git','merge','--no-edit']

    if ff == True: cmd += ['--ff-only']
    elif ff == False: cmd += ['--no-ff']

    if msg: cmd += ['-m',msg]

    cmd += [ref]

    run(cmd)


def commit(msg=None, all=False, amend=False):
    cmd = ['git','commit']
    if amend: cmd += ['--amend']
    if all: cmd += ['-a']
    if msg: cmd += ['-m',msg]
    run(cmd)
